package es.mde.gearsolid;

import java.util.AbstractMap;
import java.util.Arrays;
import java.util.Properties;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaVendorAdapter;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.transaction.annotation.EnableTransactionManagement;
import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Clase que permite configurar las persistencia de datos. Incluye especificación de archivos de
 * configuracion
 * 
 * @author Código basado en <a href="https://github.com/Awes0meM4n">Awes0meM4n</a> y modifgicado por
 *         Rodrigo de Dios García
 */
@Configuration
@PropertySource({"classpath:passwordsBD.properties", "classpath:rest.properties",
    "classpath:baseDatos.properties"})
@EnableJpaRepositories({"${repositorios}"})
@EnableTransactionManagement
public class ConfiguracionPorJava {

  @Value("${entidades}")
  String entidades;

  /**
   * Permite generar un Factory de entidades JPA en base a clases que se encuentren en la ruta
   * definida en entidades
   * 
   * Los parámetros se inyectan directamente desde Spring
   * 
   * @param dataSource
   * @param env
   * @param vendorAdapter
   * @return retorna un Factory de entidades
   */
  @Bean
  public LocalContainerEntityManagerFactoryBean entityManagerFactory(DataSource dataSource,
      Environment env, JpaVendorAdapter vendorAdapter) {

    LocalContainerEntityManagerFactoryBean em = new LocalContainerEntityManagerFactoryBean();
    em.setDataSource(dataSource);
    em.setJpaVendorAdapter(vendorAdapter);

    em.setPackagesToScan(entidades);

    Properties jpaProperties = new Properties();
    Arrays.asList("dialect", "show_sql", "hbm2ddl.auto", "enable_lazy_load_no_trans").stream()
        .map(s -> "hibernate." + s)
        .map(p -> new AbstractMap.SimpleEntry<String, String>(p, env.getProperty(p)))
        .filter(e -> e.getValue() != null)
        .forEach(e -> jpaProperties.put(e.getKey(), e.getValue()));
    em.setJpaProperties(jpaProperties);

    return em;
  }

  /**
   * Dado un Factory de Entidades genera un gestor de entidades (inyectado por Spring) genera las
   * entidades en Spring en base a las clases especificadas en el Factory
   * 
   * @param emf (inyectado por Spring)
   * @return
   */
  @Bean
  public EntityManager entityManager(EntityManagerFactory emf) {
    System.err.println("--- LAS ENTIDADES MAPEADAS SON ---");
    emf.getMetamodel().getEntities().forEach(System.err::println);
    System.err.println("----------------------------------");

    return emf.createEntityManager();
  }
  
  @Bean
  public Logger iniciarLog() {
	  return LoggerFactory.getLogger(getClass());
  }

}
